@@ -53,8 +53,6 @@
 #include "nsICookie2.h"
 #include "nsNetCID.h"
 #include "MicrobEalCertificate.h"
-#include "nsISHistoryThumbnailManager.h"
-#include "nsSHistoryThumbnailManager_contractid.h"
 #include <nsIComponentManager.h>
 #include <nsComponentManagerUtils.h>
 #include <nsILocalFile.h>
@@ -71,6 +69,8 @@
 #include "nsIPrefService.h"
 #include "nsIDOMEvent.h"
 #include "nsIDOMEventTarget.h"
+#include "nsSHistoryListener.h"
+#include "nsIAppStartup.h"
 
 static NS_DEFINE_CID(kHttpAuthManagerCID,        NS_HTTPAUTHMANAGER_CID);
 
@@ -162,12 +162,70 @@ MicrobEalObserver::UnregisterSelf(nsIComponentManager* aCompMgr,
 MicrobEalObserver::MicrobEalObserver()
 {
   mGWeb = nsnull;
+  mOssoContext = nsnull;
 }
 
 MicrobEalObserver::~MicrobEalObserver()
 {
+  if (mOssoContext)
+  {
+    osso_hw_unset_event_cb(mOssoContext, nsnull);
+    osso_deinitialize(mOssoContext);
+    mOssoContext = nsnull;
+  }
 }
 
+void
+MicrobEalObserver::OssoDisplayCallback(osso_display_state_t state, gpointer data)
+{
+  nsCOMPtr<nsIObserverService> os = do_GetService("@mozilla.org/observer-service;1");
+  if (!os)
+      return;
+
+  if (state == OSSO_DISPLAY_ON)
+      os->NotifyObservers(nsnull, "system-display-on", nsnull);
+  else
+      os->NotifyObservers(nsnull, "system-display-dimmed-or-off", nsnull);
+}
+
+void
+MicrobEalObserver::OssoHardwareCallback(osso_hw_state_t *state, gpointer data)
+{
+  NS_ASSERTION(state, "osso_hw_state_t must not be null.");
+  NS_ASSERTION(data, "data must not be null.");
+
+  osso_hw_state_t* ourState = (osso_hw_state_t*) data;
+
+  if (state->shutdown_ind) {
+    nsCOMPtr<nsIAppStartup> appService =  do_GetService("@mozilla.org/toolkit/app-startup;1");
+    if (appService)
+      appService->Quit(nsIAppStartup::eForceQuit);
+    return;
+  }
+
+  if (state->memory_low_ind) {
+      if (!ourState->memory_low_ind) {
+      nsCOMPtr<nsIObserverService> os = do_GetService("@mozilla.org/observer-service;1");
+      if (os)
+        os->NotifyObservers(nsnull, "memory-pressure", NS_LITERAL_STRING("low-memory").get());
+    }
+  }
+
+  if (state->system_inactivity_ind != ourState->system_inactivity_ind) {
+      nsCOMPtr<nsIObserverService> os = do_GetService("@mozilla.org/observer-service;1");
+      if (!os)
+        return;
+
+      if (state->system_inactivity_ind)
+          os->NotifyObservers(nsnull, "system-idle", nsnull);
+      else
+          os->NotifyObservers(nsnull, "system-active", nsnull);
+  }
+
+  memcpy(ourState, state, sizeof(osso_hw_state_t));
+}
+
+
 nsresult
 MicrobEalObserver::Init()
 {
@@ -187,6 +245,24 @@ MicrobEalObserver::Init()
   NS_ENSURE_SUCCESS(rv, rv);
   rv = prefBranch->AddObserver("microb.", this, PR_FALSE);
   NS_ENSURE_SUCCESS(rv, rv);
+
+  memset(&mHwState, 0, sizeof(osso_hw_state_t));
+
+  mOssoContext = osso_initialize("microb-engine", VERSION, PR_TRUE, nsnull);
+
+  /* Check that initilialization was ok, if not the it is still ok */
+  if (mOssoContext == nsnull)
+      return NS_OK;
+
+  osso_hw_set_event_cb(mOssoContext,
+                       nsnull,
+                       OssoHardwareCallback,
+                       &mHwState);
+
+  osso_hw_set_display_event_cb(mOssoContext,
+                               OssoDisplayCallback,
+                               nsnull);
+
   return rv;
 }
 
@@ -226,107 +302,6 @@ MicrobEalObserver::CleanAuthData ()
     return NS_OK;
 }
 
-
-nsresult 
-GetSessionHistoryThumbnailManager(nsIDOMWindow* aDOMWindow, 
-                                  nsISHistoryThumbnailManager** aSHistoryThumbnailManager,
-                                  PRInt32 aWidth, PRInt32 aHeight)
-{
-    NS_ENSURE_TRUE(aSHistoryThumbnailManager, NS_ERROR_NULL_POINTER);
-    *aSHistoryThumbnailManager = nsnull;
-
-    nsCOMPtr<nsIComponentManager> compMgr;
-    nsresult rv = NS_GetComponentManager(getter_AddRefs(compMgr));
-    NS_ENSURE_SUCCESS(rv, rv);
-    NS_ENSURE_TRUE(compMgr, NS_ERROR_NULL_POINTER);
-
-    nsCOMPtr<nsISHistoryThumbnailManager> tm; 
-    rv = compMgr->CreateInstanceByContractID(SHISTORYTHUMBNAILMANAGER_CONTRACTID,
-                                        nsnull,
-                                        NS_GET_IID(nsISHistoryThumbnailManager),
-                                        getter_AddRefs(tm));
-    NS_ENSURE_SUCCESS(rv, rv);
-    NS_ENSURE_TRUE(tm, NS_ERROR_NULL_POINTER);
-
-    const gchar *tmp_dir = g_get_tmp_dir();
-    gchar* filename = g_strdup_printf(G_MOZILLA_THUMBNAIL_FILENAME_PARAM,
-                                      tmp_dir, 
-                                      (gint)time(NULL));
-
-    nsCAutoString folder(filename);
-    rv = tm->Start(
-        aDOMWindow, 
-        folder, aWidth, aHeight);
-
-    g_free(filename);
-
-    *aSHistoryThumbnailManager = tm;
-    NS_ADDREF(*aSHistoryThumbnailManager);
-    return rv;
-}
-
-nsresult 
-RemoveAllSessionHistoryThumbnails()
-{
-#if 0
-    const gchar *tmp_dir = g_get_tmp_dir();
-    gchar* filename = g_strdup_printf(G_MOZILLA_THUMBNAILS_DIRNAME_PARAM,
-                                      tmp_dir);
-
-    if (!filename) {
-        return NS_ERROR_OUT_OF_MEMORY;
-    }
-
-    nsCAutoString folder(filename);
-    g_free(filename);
-
-    nsresult rv;
-    nsCOMPtr<nsILocalFile> localFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-    NS_ENSURE_TRUE(localFile, NS_ERROR_NULL_POINTER);
-    rv = localFile->InitWithNativePath(folder);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    rv = localFile->Remove(PR_TRUE);
-    return rv;
-#endif
-    return NS_ERROR_NOT_AVAILABLE;
-}
-
-nsresult 
-CreateSessionHistoryManager(nsIDOMWindow* aDOMWindow)
-{
-    NS_ENSURE_TRUE(aDOMWindow, NS_ERROR_NULL_POINTER);
-
-    nsresult rv = NS_ERROR_FAILURE;
-    GMozillaEngine* moz_engine = GetGMozEngineForDOMWindow(aDOMWindow);
-    if (!moz_engine)
-    {
-        return rv;
-    }
-
-    delete_session_history_manager(moz_engine);
-
-    if (!moz_engine->generate_shistory_thumbnails)
-    {
-        return rv;
-    }
-
-    nsISHistoryThumbnailManager* tm = nsnull;
-    rv = GetSessionHistoryThumbnailManager(aDOMWindow, 
-                                           &tm, 
-                                           moz_engine->shistory_thumbnail_width,
-                                           moz_engine->shistory_thumbnail_height);
-    if (NS_FAILED(rv))
-    {
-        NS_IF_RELEASE(tm);
-        tm = nsnull;
-    }
-    moz_engine->shistory_thumbnail_manager = tm;
-
-    return NS_OK; 
-}
-
 static MicrobEalXshmRenderer* EnsureXshmRenderer(GObject* aGWeb)
 {
   MicrobEalXshmRenderer *mshm = (MicrobEalXshmRenderer*)g_object_get_data(aGWeb, G_MOZILLA_PAINT_LISTENER);
@@ -335,7 +310,10 @@ static MicrobEalXshmRenderer* EnsureXshmRenderer(GObject* aGWeb)
     ULOG_DEBUG_F("%s - Creating XSHMR", __PRETTY_FUNCTION__);
 
     mshm = new MicrobEalXshmRenderer();
-    mshm->Init(aGWeb);
+    if (mshm) {
+      g_object_set_data(G_OBJECT(aGWeb), G_MOZILLA_PAINT_LISTENER, mshm);
+      mshm->Init(aGWeb);
+    }
   }
   else {
     ULOG_DEBUG_F("%s - XSHMR Exists", __PRETTY_FUNCTION__);
@@ -344,6 +322,22 @@ static MicrobEalXshmRenderer* EnsureXshmRenderer(GObject* aGWeb)
   return mshm;
 }
 
+static MicrobEalCertificate* EnsureCertListener(GObject* aGWeb)
+{
+  MicrobEalCertificate *mcert = (MicrobEalCertificate*)g_object_get_data(aGWeb, G_MOZILLA_CERTIFICATE_LISTENER);
+
+  if (!mcert) {
+    ULOG_DEBUG_F("%s - Creating CERT LIST", __PRETTY_FUNCTION__);
+    mcert = new MicrobEalCertificate();
+    g_object_set_data(G_OBJECT(aGWeb), G_MOZILLA_CERTIFICATE_LISTENER, mcert);
+  }
+  else {
+    ULOG_DEBUG_F("%s - CERT LIST Exists", __PRETTY_FUNCTION__);
+  }
+
+  return mcert;
+}
+
 /**
  * Get the current HTML document title from DOM.
  * @returns The title as a newly allocated string, or NULL on failure.
@@ -429,20 +423,14 @@ MicrobEalObserver::Observe(nsISupports *aSubject, const char *aTopic, const PRUn
   if (!strcmp(aTopic, G_MOZILLA_WEB_NAME_DEFAULT)) {
     // Set GMozilla Web
     mGWeb = aSubject;
-    // Ensure XSHM Renderer Exists
-    EnsureXshmRenderer((GObject*)mGWeb); 
 
-    // Remove all thumbnail datafiles at startup, 
-    // becasue if instance of the browser engine was crash then
-    // thumbnail-manager can't remove useless thumbnails 
-    // RemoveAllSessionHistoryThumbnails();
     return NS_OK;
   }
 
   if (!strcmp(aTopic, G_MOZILLA_ENGINE_CLEAN_AUTH)) {
     return CleanAuthData();
   }
-  
+
   if (!strcmp(aTopic, DOM_WINDOW_OPENED) && aSubject) {
     nsCOMPtr<nsIDOMWindow> win = do_QueryInterface(aSubject, &rv);
     NS_ENSURE_SUCCESS(rv, NS_OK);
@@ -451,46 +439,45 @@ MicrobEalObserver::Observe(nsISupports *aSubject, const char *aTopic, const PRUn
     // Check Engine
     if (engine) {
       // Get XSHM Renderer
-      MicrobEalXshmRenderer *xshm = EnsureXshmRenderer(engine->global);
+      MicrobEalXshmRenderer *xshm = EnsureXshmRenderer(G_OBJECT(mGWeb));
       // Check XSHM Renderer
       if (xshm)
         // Setup XSHM Renderer
         xshm->SetUp(engine);
 
-      MicrobEalCertificate *cert = new MicrobEalCertificate();
-
-      if (cert) {
+      MicrobEalCertificate *cert = EnsureCertListener(G_OBJECT(mGWeb));
+      if (cert)
         cert->SetUp(aSubject);
-        g_object_set_data(G_OBJECT(engine), G_MOZILLA_CERTIFICATE_LISTENER, cert);
+
+      nsSHistoryListener *shistory = new nsSHistoryListener();
+      if (shistory) {
+        NS_ADDREF(shistory);
+        shistory->Init(win, engine);
+        engine->shistory_listener = shistory;
       }
     }
-
-    nsCOMPtr<nsIDOMWindow> domWindow = do_QueryInterface(aSubject);
-    return CreateSessionHistoryManager(domWindow);
   }
 
   if (!strcmp(aTopic, DOM_WINDOW_CLOSED) && aSubject) {
-    MicrobEalCertificate *cert = nsnull;
     nsCOMPtr<nsIDOMWindow> win = do_QueryInterface(aSubject, &rv);
     NS_ENSURE_SUCCESS(rv, NS_OK);
 
     // Get XSHM Renderer
-    MicrobEalXshmRenderer *xshm = (MicrobEalXshmRenderer *)g_object_get_data(G_OBJECT(mGWeb), G_MOZILLA_PAINT_LISTENER);
+    MicrobEalXshmRenderer *xshm = EnsureXshmRenderer(G_OBJECT(mGWeb));
     // Check XSHM Renderer
     if (xshm)
       // UnSetup XSHM Renderer
       xshm->UnSetup(win);
 
-    // I guess this is does not work at all because engine will be always null
-    GMozillaEngine* engine = GetGMozEngineForDOMWindow(win);
-
-    if (engine) {
-      cert = (MicrobEalCertificate*)g_object_steal_data(G_OBJECT(engine), G_MOZILLA_CERTIFICATE_LISTENER);
-      delete_session_history_manager(engine);
-    }
-
+    MicrobEalCertificate *cert = EnsureCertListener(G_OBJECT(mGWeb));
     if (cert)
-      delete cert;
+      cert->UnSetup(aSubject);
+
+    return NS_OK;
+  }
+
+  if (!strcmp(aTopic, G_MOZILLA_XPCOM_SHUTDOWN)) {
+    return NS_OK;
   }
 
   if (!strcmp(aTopic, G_MOZILLA_QUIT_APPLICATION)) {
